/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import java.sql.*;
import java.util.*;

import com.ibm.etill.framework.admin.*;
import com.ibm.etill.framework.archive.*;
import com.ibm.etill.framework.cassette.*;
import com.ibm.etill.framework.*;
import com.ibm.etill.framework.log.*;
import com.ibm.etill.framework.payapi.*;
import com.ibm.etill.framework.supervisor.*;

/**
 * A <code>KitCashCleanUp</code> object is created by the KitCash cassette to clean up
 * the KitCash database tables at regular intervals.
 * <p>To get the Payment Manager timer thread to schedule hourly clean-up operations, the KitCash
 * cassette uses the following code:
 * <pre>
 *     WorkItem item = new KitCashCleanUp(60);
 *     item.doYourWork();
 * </pre>
 */
public class KitCashCleanUp implements WorkItem {

   private int interval;        // interval (in minutes) between database cleanups

   /**
    * Constructs a new KitCashCleanUp object to clean the KitCash database tables at regular intervals.
    * @param interval the number of minutes between clean-up operations
    */
   public KitCashCleanUp(int interval) {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashCleanUp.KitCashCleanUp");
      this.interval = interval;
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashCleanUp.KitCashCleanUp");
   }

   /**
    * Clears old data from the KitCash database tables and schedules another
    * cleaning operation after the pre-determined interval.
    * <p>This method is called by the Payment Manager timer thread when it is time for the clean-up
    * operation to begin.
    */
   public void doYourWork() {
      try {
         if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashCleanUp.doYourWork");

         prune();

         // Set the timer thread to re-invoke doYourWork() after the specified time has elapsed.
         long nextTime = System.currentTimeMillis() + (interval * 60000);
         Supervisor.addItemToTimerQueue(new TimeableTransaction(this, nextTime));
         System.out.println("KitCashCleanUp: next pruning: " + new java.util.Date(nextTime));
      } catch (ETillAbortOperation eao) {
         if (Trace.isAnyoneTracing())
            Trace.traceDebug(KitCashCassette.TRACEID,"Database cleanup operation failed.  Retry in " + interval + " minutes.");

         ErrorLog.logError(KitCashCassette.TRACEID, "0801", eao);
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashCleanUp.doYourWork");
   }
   
   /**
    * Returns the Cassette object associated with this WorkItem (the KitCashCassette).
    * @return Cassette The KitCashCassette.
    */
   public Cassette getCassette() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashCleanUp.getCassette");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashCleanUp.getCassette");
      return Admin.getCassetteAdminObject(KitCashCassette.TRACEID).getCassette();
   }
   
   /**
    * This method actually does the work of pruning the invalid orders from the database.
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if a database error fails.
    */
   private void prune() throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"KitCashCleanUp.prune");
      // Pruning policy:
      // Select all orders that have never moved passed REQUESTED status
      // and are at least one day old.  Delete each order along with
      // the Payment and KitCashPurchase objects associated with it.

      // We first need to calculate the timestamp for one day ago...
      Calendar c = new GregorianCalendar();
      c.setTime(new java.util.Date());
      c.add(Calendar.DAY_OF_MONTH, -1);

      String oldies = "SELECT MERCHANTNAME, ORDERNUMBER " +
                      "FROM " + ETillArchive.geteTillDatabaseOwner() + ".ETORDER " +
                      "WHERE CURRENTSTATE = " + PaymentAPIConstants.ORDER_RESET +
                      "  AND PAYMENTTYPE = 'KitCash' " +
                      "  AND TIMECREATED < {ts '" + new Timestamp(c.getTime().getTime()).toString() + "'} ";

      synchronized (ETillArchive.getArchiveLock()) {
         ETillArchive database = new ETillArchive();

         Order order = null;
         try {
            ResultSet rs = database.beginQuery(oldies);
            int count = 0;

            while (rs.next()) {
               String merchantNumber   = rs.getString("MERCHANTNAME");
               String orderNumber    = rs.getString("ORDERNUMBER");
               order = Supervisor.retrieveOrder(merchantNumber, orderNumber);
               Supervisor.getThreadCommitPoint().addToDeleteList(order);      // this will also delete the cassette order
               Supervisor.getThreadCommitPoint().addToDeleteList(order.getUniquePayment());
               Supervisor.getThreadCommitPoint().commit();
               count++;
            }
            System.out.println("\nKitCashCleanUp: delinquency pruning: " + count + " records pruned");
         } catch (SQLException e) {
            throw new ETillAbortOperation();
         } finally {
            try {database.endQuery();} catch (SQLException sqle) {ErrorLog.logError(KitCashCassette.TRACEID, "0469", sqle);}
         }
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"KitCashCleanUp.prune");
   }
}
