/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import java.io.*;
import java.net.*;
import java.util.*;

import com.ibm.etill.kitcashcassette.io.*;
import com.ibm.etill.kitcashcassette.test.card.*;

public class KitCashMessageWrapper {

   private String receiver;
   private String senderID;
   private KitCashMessage contents;
   private KitCashException error;
   private int amount = 0;

   /**
    * Constructs a new <code>KitCashMessageWrapper</code> object containing a KitCash error message.
    *
    * @param error the error message to be sent
    * @param receiver the receiver of the message
    * @param senderID the receiver's ID for the sender (e.g. order number or account number)
    */
   public KitCashMessageWrapper(String receiver, String senderID, KitCashException error) {
      this.receiver = receiver;
      this.senderID = senderID;
      this.error = error;
   }

   /**
    * Constructs a new <code>KitCashMessageWrapper</code> object containing the KitCash protocol message.
    *
    * @param contents the KitCash protocol message
    * @param receiver the receiver of the message
    * @param senderID the receiver's ID for the sender (e.g. order number or account number)
    */
   public KitCashMessageWrapper(String receiver, String senderID, KitCashMessage contents) {
      this(receiver, senderID, contents, 0);
   }

   /**
    * A private constructor.
    *
    * @param contents the KitCash protocol message
    * @param receiver the receiver of the message
    * @param senderID the receiver's ID for the sender (e.g. order number or account number)
    * @param amount the amount associated with the message
    */
   private KitCashMessageWrapper(String receiver, String senderID, KitCashMessage contents, int amount) {
      this.receiver = receiver;
      this.senderID = senderID;
      this.contents = contents;
      this.amount = amount;
   }

   /**
    * Returns the cash transfer amount from the message wrapper
    * @return the KitCash error message
    */
   public int getAmount() {
      return amount;
   }

   /**
    * Returns the KitCash message inside this wrapper
    * @return the KitCash message
    */
   public KitCashMessage getContents() {
      return contents;
   }

   /**
    * Returns the KitCash error message inside this wrapper
    * @return the KitCash error message
    */
   public KitCashException getError() {
      return error;
   }

   /**
    * Returns the ID of the receiver of the message
    * @return the ID of the receiver of the message (e.g. the merchant number of the bank ID)
    */
   public String getReceiver() {
      return receiver;
   }

   /**
    * Wraps the message or error message in a MIME message, sends it to the specified port and
    * returns the response message.
    * @param host the hostname of the receiver
    * @param port the port number of the receiver
    * @exception java.io.IOException if the communication fails
    */
   public KitCashMessageWrapper getResponse(String host, int port) throws IOException {

      KitCashMessageWrapper response = null;

      System.out.println("Attempting to open socket on host " + host + " and port " + String.valueOf(port));
      Socket socket = new Socket(host, port);
      OutputStream dos = socket.getOutputStream();
      InputStream dis = socket.getInputStream();
      System.out.println("Writing message: " + toString());
      writeMessage(dos);

      response = KitCashMessageWrapper.readMessage(dis);
      System.out.println("Response read: " + response.toString());

      if (socket != null) socket.close();
      if (dos != null) dos.close();
      if (dis != null) dis.close();

      return response;
   }

   /**
    * Returns the ID of the sender of the message as designated by the receiver.
    * <p>When sending a KitCash message from consumer to merchant, this should be the order number.
    * When sending a KitCash message from merchant to bank, this should be the account number.
    * @return the sender ID
    */
   public String getSenderID() {
      return senderID;
   }

   /**
    * Reads a MIME message from the input stream and returns the KitCashMessage contained in that stream.
    * @param is the input stream from which to read the MIME message
    * @exception java.io.IOException if the communication fails
    */
   public static KitCashMessageWrapper readMessage(InputStream is) throws IOException {
      TimeOutInputStream  tis = new TimeOutInputStream(is, MimeConst.MAX_SIZE_MIMEHEADER);
      MimeInputStream mis = new MimeInputStream(tis);

      mis.readMime();
      byte[] msgObject = mis.getMessage();

      StringTokenizer st = new StringTokenizer(new String(msgObject));
      String senderID = "";
      String receiver = "";
      int amount = 0;
      KitCashMessage msg = null;
      KitCashException error = null;
      while (st.hasMoreTokens()) {
         String pair = st.nextToken();
         String name = "";
         String value = "";
         int pos = pair.indexOf('=');
         if (pos > 0) {
            name = pair.substring(0, pos);
            value = pair.substring(pos+1);
         }

         if (name.equals("senderID")) senderID = value;
         else if (name.equals("receiver")) receiver = value;
         else if (name.equals("amount")) amount = Integer.parseInt(value);
         else if (name.equals("error")) error = new KitCashException(Short.parseShort(value));
         else if (name.equals("msg")) {
            try {
               msg = new KitCashMessage(value.getBytes());
            } catch (KitCashException e) {
               throw new IOException(KitCashDriver.text("e10")); // "Cannot create KitCashMessage object from input stream"
            }
         }
      }
      if (error != null) return new KitCashMessageWrapper(receiver, senderID, error);
      return new KitCashMessageWrapper(receiver, senderID, msg, amount);
   }

   /**
    * Sets the amount of the cash transfer in the message wrapper.
    * @param amount the amount to transfer
    */
   public void setAmount(int amount) {
      this.amount = amount;
   }

   /**
    * Returns a string representation of the <code>KitCashMessageWrapper</code>.
    * @return a string representation of the <code>KitCashMessageWrapper</code>
    */
   public String toString() {
      String s = (contents != null ? contents.toString() : error.getMessage());
      return receiver+"/"+senderID+"/'"+s+"'";
   }

   /**
    * Wraps the KitCash protocol message in a MIME message and writes it to the given output stream.
    * @param os the output stream to which to write the message
    * @exception java.io.IOException if the communication fails
    */
   public void writeMessage(OutputStream os) throws IOException {
      String content = "";

      content += "receiver=" + receiver + "\r\n";
      content += "senderID=" + senderID + "\r\n";
      if (contents != null)
         content += "msg=" + new String(contents.getMessage()) + "\r\n";
      if (error != null)
         content += "error=" + Short.toString(error.getID()) + "\r\n";
      if (amount != 0)
         content += "amount=" + amount + "\r\n";

      DataOutputStream dos = new DataOutputStream(os);
      MimeOutputStream mos = new MimeOutputStream(dos);

      mos.setContentType("application/KitCashMessage");
      mos.setLength(content.length());
      mos.setMessage(content.getBytes("8859_1"));
      mos.writeMime();

      os.flush();
   }
}
