/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import java.io.*;
import java.sql.*;
import java.util.*;

import com.ibm.etill.framework.archive.*;
import com.ibm.etill.framework.cassette.*;
import com.ibm.etill.framework.log.*;
import com.ibm.etill.framework.payapi.*;
import com.ibm.etill.framework.supervisor.*;
import com.ibm.etill.kitcashcassette.test.card.*;

/**
 * The <code>KitCashProfile</code> class is used by the KitCash cassette to read and store its configuration
 * information from the <code>KITCASHCONFIG</code> database table.  It represents an extension
 * to the framework cassette admin object.
 * <p>The one piece of information needed by the KitCash cassette is
 * the protocol port to be used when listening for incoming KitCash protocol messages from the consumer.
 */
public class KitCashProfile implements KitCashConstants, KitCashCassetteAdminConstants, FrameworkReturnCodes, Archivable {

   // Fields from the KitCashConfig database table
   static private String KITCASHCONFIG_TABLE = "KITCASHCONFIG";


   /** This keeps track of the KitCashProfile instance. */
   private static KitCashProfile profile;

   private int protocolPort;
   private static String sqlUpdate = "UPDATE " + ETillArchive.geteTillDatabaseOwner() + "." + KITCASHCONFIG_TABLE + " SET " +
                                     COL_PROTOCOLPORT + "=?";
   /**
    * Constructs a new KitCashProfile object.
    * @param rs the <code>ResultSet</code> containing a single row from the KitCashConfig database table.
    */
   private KitCashProfile(ResultSet rs) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.KitCashProfile");
      try {
         setProtocolPort(rs.getInt(COL_PROTOCOLPORT));
      } catch (SQLException e) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0098",e);
         throw new ETillCassetteException("Failed to create a KitCashProfile Object");
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.KitCashProfile");
   }

   /**
    *
    * Since cassette objects should never be created, this method should never be called.
    * It throws an exception if called.  It is required by the Archivable interface.
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the method is called.
    *
    */
   public void createRecord(Connection conn) throws ETillAbortOperation {
      throw new ETillCassetteException("Cassette objects should never be created!");
   }

   /**
    * Since cassette objects should never be deleted, this method should never be called.
    * It throws an exception if called.  It is required by the Archivable interface.
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the method is called.
    *
    */
   public void deleteRecord(Connection conn) throws ETillAbortOperation {
      throw new ETillCassetteException("Cassette objects should never be deleted!");
   }

   /**
    * Returns the protocol port that the KitCash cassette uses to listen for incoming
    * KitCash protocol messages from the consumer.
    * @return the KitCash protocol port.
    */
   public static int getProtocolPort() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.getProtocolPort");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.getProtocolPort");
      return profile.protocolPort;
   }

   /**
    * Modify this profile object with the information passed in the request.
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the update fails.
    *
    */
   protected void modifyObject(AdminRequest request, ParameterTable protocolKeywords, APIResponse response) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.modifyObject");

      /*****************************************
      *
      * Retrieve incoming fields from protocol data
      *
      *****************************************/
      Integer port = (Integer)protocolKeywords.get(PD_PROTOCOLPORT);
      setProtocolPort(port.intValue());

      /*****************************************
      *
      * Database
      *
      *****************************************/
      Supervisor.getThreadCommitPoint().addToUpdateList(this);
      Supervisor.getThreadCommitPoint().commit();

      response.setReturnCodes(PRC_OPERATION_SUCCESS, RC_NONE);

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.modifyObject");
   }

   /*****************************************
    *
    * Obtain Object
    *
    *****************************************/
   static protected KitCashProfile obtainObject() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.obtainObject");
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.obtainObject");
      return profile;
   }

   /**
    * Retrieves the KitCash configuration information from the KitCashConfig database table.
    * <p>Only one configuration record is expected on the table.  If more than one record
    * exists, then all but one will be ignored.
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the database table cannot be read or no configuration data is found.
    */
   static public void retrieveConfigRecords() throws ETillAbortOperation {

      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.retrieveConfigRecords");
      ETillArchive database = new ETillArchive();

      // Execute the given SQL statement and save the results.
      synchronized (ETillArchive.getArchiveLock()) {
         try {
            ResultSet rs = database.beginQuery("SELECT " +
                                               COL_PROTOCOLPORT +
                                               " FROM " +  ETillArchive.geteTillDatabaseOwner() + "." + KITCASHCONFIG_TABLE);

            // Determining if the result set contains a row.
            boolean tableRowExists = rs.next();
            int profileIndex = 0;

            // If a row exists then we have valid information.
            if (tableRowExists) {
               profile = new KitCashProfile(rs);
               if (Trace.isAnyoneTracing())
                  Trace.traceDebug(KitCashCassette.TRACEID,"The KitCashProfile read from " + KITCASHCONFIG_TABLE +
                                   KitCashCassette.lineSeparator +
                                   profile.toString());
            } else {
               if (Trace.isAnyoneTracing())
                  Trace.traceDebug(KitCashCassette.TRACEID,"No row was found in the " + KITCASHCONFIG_TABLE + " database table.");

               throw new Exception();
            }
         } catch (Exception e) {
            ErrorLog.logError(KitCashCassette.TRACEID,"0408", e);
            throw new ETillCassetteException("Failed to retrieve KitCash Config Records.");
         } finally {
            try {database.endQuery();} catch (SQLException sqle) {ErrorLog.logError(KitCashCassette.TRACEID,"0469", sqle);}
         }
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.retrieveConfigRecords");
   }

   /**
    * Sets the value of the protocol port where KitCash expects to receive messages from the consumer
    * @param value the port number for receiving KitCash protocol messages
    */
   private void setProtocolPort(int value) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.setProtocolPort");
      protocolPort = value;
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.setProtocolPort");
   }

   /**
    * Returns a string representation of the KitCash configuration information.
    * @return a string representation of the KitCash configuration information.
    */
   public String toString() {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.toString");

      String s = new String();

      s = COL_PROTOCOLPORT + " = " + getProtocolPort() + KitCashCassette.lineSeparator;

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.toString");
      return s;
   }

   /**
    * Updates the KitCash configuration information in the KitCashConfig database table.
    * It is required by the Archivable interface.
    * @param conn the database connection to use
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the database table cannot be updated.
    */
   public void updateRecord(Connection conn) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID, "KitCashProfile.updateRecord");

      try {
         PreparedStatement pstmt = conn.prepareStatement(sqlUpdate);
         int index = 1;
         pstmt.clearParameters();
         ETillArchive.addInteger(new Integer(protocolPort), pstmt, index++);

         if (Trace.isAnyoneTracing()) Trace.traceDatabaseWrite(KitCashCassette.TRACEID, sqlUpdate);

         pstmt.executeUpdate();
         pstmt.close();
      } catch (SQLException e) {
         ErrorLog.logError(KitCashCassette.TRACEID,"0203",e,null,null,e.toString());
         throw new ETillAbortOperation(PRC_DATABASE_ERROR, RC_NONE);
      }

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID, "KitCashProfile.updateRecord");
   }
}
