/*--------------------------------------------------------------------*/
/*  This file contains sample code.  IBM PROVIDES THIS CODE ON AN     */
/*  'AS IS' BASIS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS        */
/*  OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES  */
/*  OF MERCHANT ABILITY OR FITNESS FOR A PARTICULAR PURPOSE.          */
/*                                                                    */
/*  (C) Copyright IBM Corp. 1998, 2000                                */
/*--------------------------------------------------------------------*/
package com.ibm.etill.kitcashcassette;

import java.util.*;
import java.sql.*;

import com.ibm.etill.framework.archive.*;
import com.ibm.etill.framework.cassette.query.*;
import com.ibm.etill.framework.ETill;
import com.ibm.etill.framework.log.*;
import com.ibm.etill.framework.payapi.*;
import com.ibm.etill.framework.payserverapi.*;
import com.ibm.etill.framework.xdm.*;

/**
 * This class is used by the <code>KitCashPurchaseQueryRequest</code>
 * class to create KitCash specific objects as a result of a query request, and to combine those
 * objects with the framework ones supplied with the request.
 */
final public class PSServerKitCashPurchase implements KitCashPurchaseConstants, PaymentAPIConstants {
   private String merchantNumber;
   private String orderNumber;
   private String accountNumber;

   /**
    * Constructs a PSServerKitCashPurchase object using the result set produced
    * from the query.
    * @param rs the ResultSet produced from the query
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the construction fails.
    */
   private PSServerKitCashPurchase(ResultSet rs) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashPurchase.PSServerKitCashPurchase");
      try {
         merchantNumber = ETillArchive.readString(rs,VIEW_MERCHANTNUMBER);
         orderNumber = ETillArchive.readString(rs,VIEW_ORDERNUMBER);       
         accountNumber = ETillArchive.readString(rs,VIEW_ACCOUNTNUMBER);
      } catch (SQLException e) {
         if (Trace.isAnyoneTracing()) Trace.traceDebug(KitCashCassette.TRACEID,"PSServerKitCashPurchase(ResultSet): " + e);
         ErrorLog.logError(ETill.RB_ID, "0205", e, e.toString());
         throw new ETillAbortOperation(PRC_DATABASE_ERROR,RC_ERROR_CONNECTING_DATABASE_OR_EXEC_SQL);
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashPurchase.PSServerKitCashPurchase");
   }

   /**
    * Combines a PSServerKitCashPurchase object with a framework order object.
    * @param kitCashPurchase the PSServerKitCashPurchase object
    * @param order the PSServerOrder object
    */
   private static void combine(PSServerKitCashPurchase kitCashPurchase, PSServerOrder order) {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashPurchase.combine");

      CassetteExtensionObject object = new CassetteExtensionObject();

      object.addProperty(XDM_ACCOUNTNUMBER, kitCashPurchase.accountNumber, null);

      order.setCassetteExtensionObject(object);

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashPurchase.combine");
   }

   /**
    * Combines a list of PSServerKitCashPurchase objects with a list of framework purchase (order) objects.
    * @param kitCashPurchases the list of PSServerKitCashPurchase objects
    * @param purchases the list of purchase (order) objects
    */
   public static void combineQueryResults(Enumeration kitCashPurchases, Enumeration purchases) {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashPurchase.combineQueryResults");

      /*****************************************
      *  Build a lookup table for all purchases
      *****************************************/

      Hashtable purchaseTable = new Hashtable();
      while (kitCashPurchases.hasMoreElements()) {
         PSServerKitCashPurchase kitCashPurchase = (PSServerKitCashPurchase) kitCashPurchases.nextElement();
         OrderKey key = new OrderKey(kitCashPurchase.merchantNumber,kitCashPurchase.orderNumber);
         purchaseTable.put(key,kitCashPurchase);
      }

      /*****************************************
      *  Iterate through the orders and bind them to kitCashPurchases
      *****************************************/

      while (purchases.hasMoreElements()) {
         PSServerOrder order = (PSServerOrder) purchases.nextElement();
         OrderKey key = new OrderKey(order.getMerchantNumber(),order.getOrderNumber());
         PSServerKitCashPurchase kitCashPurchase = (PSServerKitCashPurchase) purchaseTable.get(key);
         if (kitCashPurchase != null) combine(kitCashPurchase,order);
      }

      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashPurchase.combineQueryResults");
   }

   /**
    * Creates PSServerKitCashPurchase objects from each item in the result set.
    * @param rs the the ResultSet
    * @exception com.ibm.etill.framework.payapi.ETillAbortOperation
    *            if the query fails.
    */
   public static Vector query(ResultSet rs) throws ETillAbortOperation {
      if (Trace.isAnyoneTracing()) Trace.traceFunctionEntry(KitCashCassette.TRACEID,"PSServerKitCashPurchase.query");
      Vector result = new Vector();

      try {
         while (rs.next()) {
            PSServerKitCashPurchase newPurchase = new PSServerKitCashPurchase(rs);
            result.addElement(newPurchase);
         }
      } catch (SQLException e) {
         ErrorLog.logError(ETill.RB_ID, "0205", e, e.toString());
         throw new ETillAbortOperation(PRC_DATABASE_ERROR,RC_ERROR_CONNECTING_DATABASE_OR_EXEC_SQL);
      }
      if (Trace.isAnyoneTracing()) Trace.traceFunctionExit(KitCashCassette.TRACEID,"PSServerKitCashPurchase.query");
      return result;
   }
}
